from gpiozero import Button
from gpiozero import Motor
import threading
import signal
import variables as v


class SetAssemblyLength(threading.Thread):
    __motor_up_speed = v.motor_up_speed
    __motor_down_speed = v.motor_down_speed
    __sensor_circumference = v.sensor_circumference
    __sensor_steps = v.sensor_steps
    
    def __init__(self, s_pin_a, s_pin_b, m_pin_a, m_pin_b, thread_id):
        threading.Thread.__init__(self)

        self.daemon = True
        self.__pin_a = Button(s_pin_a)

        self.__pin_b = Button(s_pin_b)
        self.__set_length = v.start_length
        self.__motor = Motor(m_pin_a, m_pin_b, pwm=True)
        self.__diff = 0
        self.__length = v.start_length
        self.__thread_id = thread_id

    def run(self):  # Starts the motors when the tread is started
        self.__start_motor()

    def __move_motor(self):  # Decides what the motor should do based on difference between set and current length

        if self.__diff < -v.error:
            self.__motor.backward(self.__motor_up_speed)

        elif self.__diff > v.error:
            self.__motor.forward(self.__motor_down_speed)

        elif (self.__diff > -v.error) & (self.__diff < v.error):
            self.__motor.stop()

    def __pin_a_rising(self):  # Pin A event handler
        if self.__pin_b.is_pressed:
            self.__length = self.__length - self.__sensor_circumference / self.__sensor_steps
            self.__diff = self.__set_length - self.__length
            self.__move_motor()
            # print(self.length, thread_id)

    def __pin_b_rising(self):  # Pin B event handler
        if self.__pin_a.is_pressed:
            self.__length = self.__length + self.__sensor_circumference / self.__sensor_steps
            self.__diff = self.__set_length - self.__length
            self.__move_motor()
            # print(self.length, thread_id)

    def __start_motor(self):  # Starts the sensor event handler and starts motor movement

        self.__pin_a.when_pressed = self.__pin_a_rising  # Register the event handler for pin A
        self.__pin_b.when_pressed = self.__pin_b_rising  # Register the event handler for pin B

        self.__diff = self.__set_length - self.__length
        self.__move_motor()
        signal.pause()

    def new_length(self, s_length):  # Used to set the length of the rope

        self.__set_length = s_length
        self.__diff = self.__set_length - self.__length
        self.__move_motor()
        print(s_length, self.__thread_id)
